/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.validation;

import static org.ow2.petals.se.validation.Constants.FILTER_OPERATION;
import static org.ow2.petals.se.validation.Constants.VALIDATE_OPERATION;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.ow2.petals.commons.exception.ExceptionUtil;
import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.se.validation.model.ValidationConfigurationHandler;
import org.ow2.petals.se.validation.service.ValidationService;

/**
 * The JBI Listener for the VALIDATION service engine.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author Mathias Beldame - EBM WebSourcing
 * @author Vincent Zurczak - EBM WebSourcing
 */
public class ValidationJBIListener extends AbstractJBIListener {

    /**
     * A prefix to use for logged messages.
     * <p>
     * This prefix is updated for every processed message.<br />
     * It only contains the exchange ID.
     * </p>
     */
    private String logHint;

    /**
     * An instance of validation service.
     */
    private ValidationService validationService;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.listener.AbstractListener #init()
     */
    @Override
    public void init() {
        this.validationService = new ValidationService(getLogger());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.listener.AbstractJBIListener
     * #onJBIMessage(org.ow2.petals.component.framework.api.message.Exchange)
     */
    @Override
    public boolean onJBIMessage(final Exchange exchange) {
        if (exchange.isActiveStatus()) {
            Logger log = getLogger();
            this.logHint = "Exchange " + exchange.getExchangeId();

            if (log.isLoggable(Level.FINE))
                log.fine(this.logHint + " was received and is started to be processed.");

            // Validate the exchange
            if (!validateExchange(exchange)) {
                if (log.isLoggable(Level.WARNING))
                    log.warning(this.logHint + " encountered a problem. "
                            + exchange.getError().getMessage());
            }

            // Process the exchange
            else {
                try {
                    this.processExchange(exchange);
                    if (log.isLoggable(Level.FINE))
                        log.fine(this.logHint + " was successfully processed.");

                } catch (PEtALSCDKException e) {

                    if (log.isLoggable(Level.WARNING))
                        log.warning(this.logHint + " encountered a problem. " + e.getMessage());

                    // Technical error, it would be set as a Fault by the CDK
                    if (exchange.isActiveStatus()) {
                        Exception exception = new MessagingException(
                                ExceptionUtil.getExtendedMessage(e));
                        exchange.setError(exception);
                    }

                } catch (MessagingException e) {

                    if (log.isLoggable(Level.WARNING))
                        log.warning(this.logHint + " encountered a problem. " + e.getMessage());

                    // Technical error, it would be set as a Fault by the CDK
                    if (exchange.isActiveStatus())
                        exchange.setError(e);
                }
            }
        }
        return true;
    }

    /**
     * Validates the operation and the MEP.
     * 
     * @param exchange
     *            the message exchange
     * @return true if the exchange can be processed, false otherwise
     */
    private boolean validateExchange(Exchange exchange) {

        boolean result = true;
        final QName operation = exchange.getOperation();

        // Null operation
        if (operation == null) {
            exchange.setError(new Exception(
                    "The JBI operation must be set by the consumer of the service."));
            result = false;
        }

        // The "validate" operation
        else if (VALIDATE_OPERATION.equals(operation.getLocalPart().toLowerCase())) {
            if (!Message.MEPConstants.IN_OUT_PATTERN.equals(exchange.getPattern())) {
                String msg = "The operation '" + operation + "' must use the pattern 'InOut'.";
                exchange.setError(new Exception(msg));
                result = false;
            }
        }

        // The "filter" operation
        else if (FILTER_OPERATION.equals(operation.getLocalPart().toLowerCase())) {
            if (!Message.MEPConstants.IN_OUT_PATTERN.equals(exchange.getPattern())) {
                String msg = "The operation '" + operation + "' must use the pattern 'InOut'.";
                exchange.setError(new Exception(msg));
                result = false;
            }
        }

        // Unsupported operation
        else {
            String msg = "The operation '" + operation + "' is not supported by this component.";
            exchange.setError(new Exception(msg));
            result = false;
        }

        return result;
    }

    /**
     * Processes the validation and sets the response as Source or attachment.
     * 
     * @param exchange
     *            the exchange to process
     * @throws MessagingException
     *             if the exchange properties could not be read
     * @throws PEtALSCDKException
     *             if the response could not be created
     */
    private void processExchange(Exchange exchange) throws MessagingException, PEtALSCDKException {

        Logger log = getLogger();
        if (log.isLoggable(Level.FINE))
            log.fine(this.logHint
                    + ": the request was validated, the transformation is about to start.");

        // Get the configuration handler
        ValidationConfigurationHandler handler = ((ValidationSe) this.component)
                .getValidationConfigurationHandler(exchange.getEndpointName());

        if (handler == null) {
            String msg = this.logHint
                    + " encountered a problem. No handler was found for this end-point.";
            if (log.isLoggable(Level.SEVERE))
                log.severe(msg);

            throw new PEtALSCDKException(msg);
        }

        // Dispatch the invocation
        String opName = exchange.getOperation().getLocalPart().toLowerCase();
        if (VALIDATE_OPERATION.equals(opName))
            this.validationService.validate(exchange, handler.getSchema());
        else
            this.validationService.filter(exchange, handler.getSchema());
    }
}
