/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.validation.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.XMLConstants;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.se.validation.util.XsdResourceResolver;
import org.xml.sax.SAXException;

/**
 * A handler for Validation configurations.
 * <p>
 * Every element derived from the SU configuration and required at runtime
 * is hold by such a handler.
 * </p>
 * 
 * @author Vincent Zurczak - EBM WebSourcing
 */
public class ValidationConfigurationHandler {

	/**
	 * The Validation configuration.
	 */
	private final ValidationConfiguration validationConfiguration;

	/**
	 * The XML schema instance, built from the XSD files from the SU.
	 */
	private Schema schema;

	/**
	 * An object that will be used as a lock in the management of the instance's state.
	 */
	private final Object lock = new Object();



	/**
	 * Constructor.
	 * @param validationConfiguration a Validation configuration
	 * @param logger the component's logger
	 */
	public ValidationConfigurationHandler( ValidationConfiguration validationConfiguration, Logger logger ) {
		this.validationConfiguration = validationConfiguration;
	}


	/**
	 * Starts the handler.
	 * <p>
	 * An XSL transformer is created from the XSL style sheet and cached.<br />
	 * If the SU embeds JAR files, they are loaded in the class loader of the handler,
	 * so that they can be used by the XSL transformation.
	 * </p>
	 * 
	 * @param logger the component's logger
	 * @throws PEtALSCDKException if the handler could not be started
	 */
	public void start( Logger logger ) throws PEtALSCDKException {

		// Try to get the XSD from the file system
		synchronized( this.lock ) {

			String suRoot = this.validationConfiguration.getSuInstallRoot();
			String logPrefix = this.validationConfiguration.getServiceUnitName();
			InputStream xsdInputStream = null;
			try {
				File xsdFile = new File( suRoot, this.validationConfiguration.getXsdPath());
				xsdInputStream = new FileInputStream( xsdFile );

				Source source = new StreamSource( xsdInputStream );
				SchemaFactory schemaFactory = SchemaFactory.newInstance( XMLConstants.W3C_XML_SCHEMA_NS_URI );
				XsdResourceResolver resolver = new XsdResourceResolver( suRoot, logger );
				schemaFactory.setResourceResolver( resolver );
				this.schema = schemaFactory.newSchema( source );

			} catch (final FileNotFoundException e) {
				throw new PEtALSCDKException( logPrefix + ": failed to retrieve the XSD resources.", e );

			} catch( SAXException e ) {
				throw new PEtALSCDKException( logPrefix + ": failed to create to a schema instance.", e );

			} finally {
				// Close the input stream
				if( xsdInputStream != null ) {
					try {
						xsdInputStream.close();

					} catch( IOException e ) {
                        if (logger.isLoggable(Level.WARNING))
                            logger.warning(logPrefix + ": the XSD input stream could not be closed. Error discarded.");
					}
				}
			}
		}
	}


	/**
	 * Stops the handler.
	 * <p>
	 * The schema instance is set to null.
	 * </p>
	 */
	public void stop() {

		synchronized( this.lock ) {
			this.schema = null;
		}
	}


	/**
	 * @return the schema
	 */
	public Schema getSchema() {
		return this.schema;
	}


	/**
	 * @return the associated end-point name
	 * @see org.ow2.petals.se.validation.model.ValidationConfiguration
	 * #getEndpointName()
	 */
	public String getEndpointName() {
		return this.validationConfiguration.getEndpointName();
	}
}
