/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.validation.service;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;

import org.apache.commons.lang.CharEncoding;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;
import com.ebmwebsourcing.easycommons.stream.InputStreamForker;

/**
 * The Validation service.
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author Mathias Beldame - EBM WebSourcing
 */
public class ValidationService {

    /**
     * The component's logger.
     */
    private final Logger logger;

    /**
     * Constructor.
     * 
     * @param logger
     *            the component's logger
     */
    public ValidationService(final Logger logger) {
        this.logger = logger;
    }

    /**
     * Implementation of the <b>filter</b> operation.
     * <p>
     * If the message is valid, it is forwarded on the OUT. Otherwise, a Fault
     * is set.
     * </p>
     * 
     * @param exchange
     *            the exchange
     * @param xsdSchema
     *            the XML schema instance
     * @throws MessagingException
     *             if the fault could be set on the exchange
     * @throws PEtALSCDKException
     *             if the fault could not be created
     */
    public void filter(Exchange exchange, Schema xsdSchema) throws MessagingException,
            PEtALSCDKException {

        Source responseSource = null;
        String logHint = "Exchange " + exchange.getExchangeId();
        Source source = exchange.getInMessageContentAsSource();
        if (source instanceof StreamSource) {
            InputStreamForker messageAsStream = new InputStreamForker(
                    ((StreamSource) source).getInputStream());
            try {
                source = new StreamSource(messageAsStream.fork());
                responseSource = new StreamSource(messageAsStream.fork());
            } catch (final IOException e) {
                throw new UncheckedException(e);
            }
        } else {
            responseSource = source;
        }

        try {
            xsdSchema.newValidator().validate(source);
            if (this.logger.isLoggable(Level.FINE))
                this.logger.fine(logHint + ": the XML message is valid.");

            exchange.setOutMessageContent(responseSource);

        } catch (SAXException e) {
            if (this.logger.isLoggable(Level.WARNING))
                this.logger.warning(logHint + ": the XML message is invalid. " + e.getMessage());

            setFilterFault(exchange, e.getMessage());

        } catch (IOException e) {
            if (this.logger.isLoggable(Level.WARNING))
                this.logger.warning(logHint + ": the XML message is invalid. " + e.getMessage());

            setFilterFault(exchange, e.getMessage());
        }
    }

    /**
     * Sets a Fault for the "filter" operation.
     * 
     * @param exchange
     *            the exchange
     * @param errorMessage
     *            the error message
     * @throws MessagingException
     *             if the fault could be set on the exchange
     * @throws PEtALSCDKException
     *             if the fault could not be created
     */
    private void setFilterFault(Exchange exchange, String errorMessage) throws MessagingException,
            PEtALSCDKException {

        StringBuffer faultMessage = new StringBuffer();
        faultMessage
                .append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        faultMessage.append("<validationFault xmlns='");
        faultMessage.append(exchange.getOperation().getNamespaceURI());
        faultMessage.append("'>");
        faultMessage.append("<message>");
        faultMessage.append(errorMessage);
        faultMessage.append("</message></validationFault>");

        Fault fault = exchange.createFault();
        Source source = SourceUtil.createSource(faultMessage.toString(),
                CharEncoding.UTF_8);
        fault.setContent(source);
        exchange.setFault(fault);

    }

    /**
     * Implementation of the <b>validate</b> operation.
     * <p>
     * If message is valid, it is forwarded on the OUT. Otherwise, a Fault is
     * set.
     * </p>
     * 
     * @param exchange
     *            the exchange
     * @param xsdSchema
     *            the XML schema instance
     * @throws MessagingException
     *             if the fault could be set on the exchange
     * @throws PEtALSCDKException
     *             if the fault could not be created
     */
    public void validate(Exchange exchange, Schema xsdSchema) throws MessagingException,
            PEtALSCDKException {

        // Get in Message content
        String logHint = "Exchange " + exchange.getExchangeId();
        Source inContent = exchange.getInMessageContentAsSource();
        try {
            xsdSchema.newValidator().validate(inContent);
            if (this.logger.isLoggable(Level.FINE))
                this.logger.fine(logHint + ": the XML message is valid.");

            setValidateResponseOk(exchange);

        } catch (SAXException e) {
            if (this.logger.isLoggable(Level.WARNING))
                this.logger.warning(logHint + ": the XML message is invalid. " + e.getMessage());

            setValidateResponseKo(exchange, e.getMessage());

        } catch (IOException e) {
            if (this.logger.isLoggable(Level.WARNING))
                this.logger.warning(logHint + ": the XML message is invalid. " + e.getMessage());

            setValidateResponseKo(exchange, e.getMessage());
        }
    }

    /**
     * Sets the KO response for the "validate" operation.
     * 
     * @param exchange
     *            the exchange
     * @param errorMessage
     *            the error message
     * @throws MessagingException
     *             if the fault could be set on the exchange
     * @throws PEtALSCDKException
     *             if the fault could not be created
     */
    private void setValidateResponseKo(Exchange exchange, String errorMessage)
            throws MessagingException, PEtALSCDKException {

        StringBuffer responseMessage = new StringBuffer();
        responseMessage
                .append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><tns:validateResponse xmlns:tns='");
        responseMessage.append(exchange.getOperation().getNamespaceURI());
        responseMessage.append("'><tns:valid>false</tns:valid><tns:comment>");
        responseMessage.append(errorMessage);
        responseMessage.append("</tns:comment></tns:validateResponse>");

        Source source = SourceUtil.createSource(responseMessage.toString(),
                CharEncoding.UTF_8);
        exchange.setOutMessageContent(source);
    }

    /**
     * Sets the OK response for the "validate" operation.
     * 
     * @param exchange
     *            the exchange
     * @throws MessagingException
     *             if the fault could be set on the exchange
     * @throws PEtALSCDKException
     *             if the fault could not be created
     */
    private void setValidateResponseOk(Exchange exchange) throws MessagingException,
            PEtALSCDKException {

        StringBuffer responseMessage = new StringBuffer();
        responseMessage
                .append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><tns:validateResponse xmlns:tns='");
        responseMessage.append(exchange.getOperation().getNamespaceURI());
        responseMessage.append("'><tns:valid>true</tns:valid></tns:validateResponse>");

        Source source = SourceUtil.createSource(responseMessage.toString(),
                CharEncoding.UTF_8);
        exchange.setOutMessageContent(source);
    }
}
